<?php

namespace Drupal\mcapi_credcom_leaf;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\Query\QueryInterface;


class SettingsForm extends ConfigFormBase {

  private $moduleHandler;
  private $traded;

  /**
   * {@inheritdoc}
   */
  public function getFormID() {
    return 'mcapi_credcom_leaf_settings_form';
  }

  public function __construct(ConfigFactoryInterface $configFactory, $module_handler, QueryInterface $transaction_entity_query) {
    $this->setConfigFactory($configFactory);
    $this->moduleHandler = $module_handler;
    $this->traded = (bool)$transaction_entity_query->accessCheck(FALSE)->condition('type', 'remote')->execute();
  }

  static public function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('entity_type.manager')->getStorage('mcapi_transaction')->getQuery()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('mcapi_credcom_leaf.settings');

    $form['curr_id'] = [
      '#title' => $this->t('Currency used for intertrading'),
      '#type' => 'mcapi_currency_select',
      '#multiple' => FALSE,
      '#default_value' => $config->get('curr_id'),
      '#weight' => 0,
      '#access'=> !$this->traded,
      '#required' => TRUE
    ];
    $form['wallet_name'] = [
      '#title' => $this->t('Friendly name of the intertrading wallet'),
      '#description' => $this->t("The wallet's name as it appears to users"),
      '#type' => 'textfield',
      '#default_value' => $config->get('wallet_name'),
      '#weight' => 2,
      '#required' => TRUE
    ];


    if ($url = $config->get('rootwards_node_url')) {
      $connected = mcapi_credcom_handshake($url);
    }

    $form['connection'] = [
      '#title' => $this->t('Connection to rootwards node'),
      '#type' => 'details',
      '#open' => !$connected,
      '#weight' => 4,
      'node_name' => [
        '#title' => $this->t('The name of the current site on the network'),
        '#description' => t('This cannot be changed once connected'),
        '#type' => 'textfield',
        '#default_value' => $config->get('node_name'),
        '#placeholder' => $this->t('lowercase with _ and no spaces.'),
        '#element_validate' => ['::machineName'],
        '#weight' => 1,
        '#disabled'=> $connected,
        '#required' => TRUE
      ],
      'rootwards_node_url' => [
        '#title' => $this->t('Rootwards node URL'),
        '#type' => 'textfield',
        '#default_value' => $config->get('rootwards_node_url', $this->defaultSiteName()),
        '#element_validate' => ['::handshake'],
        '#disabled'=> $connected,
        '#required' => TRUE,
        '#weight' => 2
      ]
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable('mcapi_credcom_leaf.settings')
      ->set('rootwards_node_url', $form_state->getValue('rootwards_node_url'))
      ->set('wallet_name', $form_state->getValue('wallet_name'))
      ->set('curr_id', $form_state->getValue('curr_id'))
      ->set('node_name', $form_state->getValue('node_name'))
      ->save();
  }

  protected function getEditableConfigNames() {
    return ['mcapi_credcom_leaf.settings'];
  }


  /**
   * Element validation callback for exchange id on clearing central
   *
   * Value must be 3 letters then 4 numbers.
   */
  function handshake(&$element, $form_state) {
    $url = $element['#value'];
    if (!mcapi_credcom_handshake($url)) {
      $form_state->setError($element, t('This node is not connected to %url.'));
    }
  }
  /**
   * Element validation callback for exchange id on clearing central
   *
   * Value must be 3 letters then 4 numbers.
   */
  private function machineName(&$element, $form_state) {
    $name = $element['#value'];
    if (!preg_match('/[a-z_]/', $element['#value'])) {
      $form_state->setError($element, t('Invalid node name'));
    }
  }

  private function defaultSiteName() {
    $site_name = $this->config('system.site')->get('site_name');
    return preg_replace('/[^a-z]/', '_', strtolower($site_name));
  }


}
