<?php

namespace Drupal\mcapi\EventSubscriber;

use Drupal\mcapi\Entity\Currency;
use Drupal\mcapi\Entity\Wallet;
use Drupal\user\Entity\Role;
use Drupal\field\Entity\FieldConfig;
use Drupal\migrate\Plugin\Migration;
use Drupal\migrate\Event\MigrateImportEvent;
use Drupal\migrate\Event\MigratePreRowSaveEvent;
use Drupal\migrate\MigrateSkipRowException;
use Drupal\mcapi\EventSubscriber\MigrationSubscriber;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Alterations to migrations
 */
class MigrationSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      'migrate.pre_import' => [['migratePreImport']],
      'migrate.post_import' => [['migratePostImport']],
      'migrate.pre_row_save' => [['migratePreRowSave']]
    ];
  }

  /**
   * Subscribed event callback.
   *
   * @param MigratePreRowSaveEvent $event
   *
   * Change the name of the transaction description field, which was a variable
   * in d7, but an entity basefield in d8. Change the names of the transaction
   * entitytype and bundle.
   */
  public function migratePreRowSave(MigratePreRowSaveEvent $event) {
    $row = $event->getRow();
    $migration = $event->getMigration();

    // This is a bit blunt but efficient.
    if ($row->getSourceProperty('entity_type') == 'transaction') {
      $row->setDestinationProperty('entity_type', 'mcapi_transaction');
      $row->setDestinationProperty('bundle', 'mcapi_transaction');
      $row->setDestinationProperty('targetEntityType', 'mcapi_transaction');
    }

        // Don't migrate worth and transaction field definitions
    $field_migrations = ['d7_field','d7_field_instance', 'd7_field_formatter_settings', 'd7_field_instance_widget_settings', 'uc7_field'];
    if (in_array($migration->id(), $field_migrations)) {
      if ($row->getSourceProperty('entity_type') == 'transaction') {
        $existing_fields = ['worth', MigrationSubscriber::d7DescriptionFieldname($migration)];
        if (in_array($row->getSourceProperty('field_name'), $existing_fields)) {
          throw new MigrateSkipRowException('Already installed');
        }
      }
    }
    // Don't bother migrating transaction view modes.
    if ($migration->getSourcePlugin()->getPluginId() == 'd7_view_mode' and $row->getSourceProperty('entity_type') == 'transaction') {
      throw new MigrateSkipRowException('Transaction view modes already installed');
    }

    // Rename the transaction description field
    if ($migration->id() == 'd7_mcapi_transaction') {
      $old_desc_field_name = $this->d7DescriptionFieldname($migration);
      // Rename the transaction description field
      $row->setDestinationProperty(
        'description',
        $row->getSourceProperty($old_desc_field_name)
      );
      $row->removeDestinationProperty($old_desc_field_name);
    }
  }

  /**
   * Subscribed event callback.
   */
  function migratePreImport(MigrateImportEvent $event) {
    // Delete existing currencies before importing from the old site.
    if ($event->getMigration()->id() == 'd7_mcapi_currency') {
      foreach (Currency::loadMultiple() as $currency) {
        if ($currency->access('delete')) {
          $currency->delete();
        }
      }
    }
    if ($event->getMigration()->id() == 'd7_user') {
      // Delete wallets to ensure no clash with incoming wallets
      foreach (Wallet::loadMultiple() as $wallet) {
        $wallet->delete();
      }
      // Disable auto adding wallets
      $this->setWalletUserAutowallet(FALSE);
    }
  }

  private function setWalletUserAutowallet(bool $switch) {
    $wallet_holder_config = FieldConfig::load('mcapi_wallet.mcapi_wallet.holder');
    $user_settings = $wallet_holder_config->getSetting('user');
    if (!$switch) {
      \Drupal::state()->set('migrate_autowallet', $user_settings['autowallet']);
      $user_settings['autowallet'] = FALSE;
    }
    elseif ($switch) {
      $user_settings['autowallet'] = \Drupal::state()->get('migrate_autowallet');
      \Drupal::state()->delete('migrate_autowallet');
    }
    $wallet_holder_config->setSetting('user', $user_settings)->save();
  }

  /**
   * Subscribed event callback.
   * Set the autoadd back to what it was before the user migration
   */
  function migratePostImport(MigrateImportEvent $event) {
    if ($event->getMigration()->id() == 'd7_user') {
      // Restore auto adding wallets
      $this->setWalletUserAutowallet(TRUE);
    }
    // Finish the permission migration, because it didn't map 1:1
    elseif ($event->getMigration()->id() == 'd7_user_roles') {
      foreach (Role::loadMultiple() as $role) {
        if ($role->hasPermission('create credit transaction')) {
          $role->grantPermission('create bill transaction');
          $role->grantPermission('view all wallets');
          $role->grantPermission('view all transactions');
        }
        if ($role->hasPermission('manage mcapi')) {
          $role->grantPermission('create 3rdparty transaction');
          $role->grantPermission('create wallets');
        }
        $role->save();
      }
    }
    // Replace any d7 users that were deleted with dummy wallets
    elseif ($event->getMigration()->id() == 'd7_mcapi_transaction') {
      $all_wids = \Drupal::database()->select('mcapi_transaction_totals', 't')
        ->fields('t', ['wid'])
        ->execute()->fetchCol();
      $existing_wids = \Drupal::database()->select('mcapi_wallet', 'w')
        ->fields('w', ['wid'])
        ->execute()->fetchCol();
      $user1 = \Drupal\user\Entity\User::load(1);
      foreach (array_diff($all_wids, $existing_wids) as $wid) {
        Wallet::Create([
          'name' => t("D7 Deleted user @wid", ['@wid' => $wid]),
          'holder' => $user1,
          'wid' => $wid
        ]);
      }
    }
  }

  /**
   * Utility.
   *
   * @param Migration $migration
   *
   * @return string
   *   The fieldname of the d7 transaction description
   */
  static function d7DescriptionFieldname(Migration $migration) {
    $result =  $migration->getSourcePlugin()
     ->getDatabase()
     ->select('variable', 'v')
     ->fields('v', ['value'])
     ->condition('name', 'transaction_description_field')
     ->execute()
     ->fetchField();
   return unserialize($result);
  }

}
