<?php

namespace Drupal\mcapi\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection;

/**
 * Provide default Wallet selection handler.
 *
 * The settings allow the wallet selection to be restricted to wallets the user owns.
 *
 * @EntityReferenceSelection(
 *   id = "default:mcapi_wallet",
 *   label = @Translation("Wallet selection"),
 *   entity_types = {"mcapi_wallet"},
 *   group = "default"
 * )
 *
 */
class WalletSelection extends DefaultSelection {

  public function defaultConfiguration() {
    $vals = parent::defaultConfiguration();
    $vals['my_wallets_only'] = FALSE;
    return $vals;
  }

  /**
   * {@inheritdoc}
   *
   * @note This override avoids having to load a large number of wallets into memory just to get the titles.
   *
   * @note this runs BEFORE hook_form_alter
   */
  public function getReferenceableEntities($match = NULL, $match_operator = 'CONTAINS', $limit = 0) {
    $entities = [];
    $query = $this->buildEntityQuery($match, $match_operator);
    if ($limit > 0) {
      $query->range(0, $limit);
    }
    if ($wids = $query->execute()) {
      $wallets = \Drupal::database()
        ->select('mcapi_wallet', 'w')
        ->fields('w', ['wid', 'name'])
        ->condition('wid', $wids, 'IN')
        ->execute()
        ->fetchAllKeyed(0);
      $entities['mcapi_wallet'] = array_map(['Drupal\Component\Utility\Html', 'escape'], $wallets);
    }
    return $entities;
  }

  /**
   * {@inheritdoc}
   */
  public function validateReferenceableEntities(array $ids) {
    $valid_ids = $this->buildEntityQuery()->execute();
    // User 1 skips validation. this is helpful for importing.
    if ($this->currentUser->id() != 1) {
      $ids = array_intersect($ids, $valid_ids);
    }
    return $ids;
  }

  /**
   * Identify wallet IDs based on the string and direction.
   *
   * @param string $match
   *   A fragment of the wallet name.
   *
   * @return Drupal\Core\Entity\Query\Sql\Query
   *   An entity query object, unexecuted
   */
  public function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = $this->entityTypeManager->getStorage('mcapi_wallet')->getQuery()->accessCheck(TRUE);
    if ($match) {
      $query->condition('name', $match, $match_operator);
    }
    if ($this->configuration['my_wallets_only']) {
      $query->condition('holder.target_type', 'user');
      $query->condition('holder.target_id', $this->currentUser->id(), 'IN');
    }
    return $query;
  }

  /**
   * This is only called in MassPay::buildEntity
   *
   * @param array $ids
   *
   * @return array
   *   All the wallets not passed
   */
  public function inverse(array $ids = []) {
    $entities = $this->getReferenceableEntities();
    return array_diff(array_keys($entities['mcapi_wallet']), $ids);
  }
}
