<?php

namespace Drupal\mcapi\Plugin\Field\FieldWidget;

use Drupal\mcapi\Form\TransactionForm;
use Drupal\mcapi\Plugin\EntityReferenceSelection\WalletSelection;
use Drupal\Core\Field\Plugin\Field\FieldWidget\EntityReferenceAutocompleteWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginManager;
use Symfony\Component\Validator\ConstraintViolationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the wallet selector widget.
 *
 * @FieldWidget(
 *   id = "wallet_reference_autocomplete",
 *   label = @Translation("Wallet reference"),
 *   description = @Translation("Select from all wallets"),
 *   field_types = {
 *     "wallet_reference",
 *     "burser_reference"
 *   }
 * )
 */
class WalletReferenceAutocompleteWidget extends EntityReferenceAutocompleteWidget {

  protected $maxSelect;

  /**
   * @var SelectionPluginManager
   */
  protected $entityRefSelectionManager;

  public function __construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings, ConfigFactory $config_factory, SelectionPluginManager $entity_ref_selection) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->maxSelect = $config_factory->get('mcapi.settings')->get('max_select');
    $this->entityRefSelectionManager = $entity_ref_selection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('plugin.manager.entity_reference_selection'),
    );
  }

  /**
   * {@inheritdoc}
   *
   * For 1st party forms, the widget needs to know whether the payment is
   * incoming or outgoing in order to determine the referencable wallets for
   * each of the payer and payee.
   *
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $handler_settings = $this->fieldDefinition->getSetting('handler_settings');
    if (!$form_state->has('transaction_direction')) {
      return parent::formElement($items, $delta, $element, $form, $form_state);
    }

    $dir = $form_state->get('transaction_direction');
    if (
      ($this->fieldDefinition->getName() == 'payee' and $dir == TransactionForm::DIR_INCOMING)
      or
      ($this->fieldDefinition->getName() == 'payer' and $dir == TransactionForm::DIR_OUTGOING)
    ) {
      $this->fieldDefinition->setSetting('handler_settings', ['my_wallets_only' => TRUE]);
      $entities = $this->entityRefSelectionManager
        ->getSelectionHandler($this->fieldDefinition)
        ->getReferenceableEntities();
      $wallets = $entities['mcapi_wallet'];
      if (count($wallets) == 1) {
        $element += [
          '#type' => 'hidden',
          '#value' => key($wallets)
        ];
      }
      elseif (count($wallets) > 1) {
        $element += [
          '#type' => 'select',
          '#options' => $wallets,
          '#default_value' => key($wallets)
        ];
      }
      else {
        trigger_error('No wallets available', E_USER_ERROR);
      }
    }
    else {
      $element += [
        '#type' => 'wallet_entity_auto', //this is just a wrapper around element entity_autocomplete
        '#target_type' => 'mcapi_wallet',
        '#selection_settings' => $this->getFieldSetting('handler_settings'),
        '#default_value' => $items->referencedEntities()[$delta] ?? NULL,
        '#placeholder' => $this->getSetting('placeholder'),
        '#size' => $this->getSetting('size')
      ];
    }

    return ['target_id' => $element];
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $error, array $form, FormStateInterface $form_state) {
    return isset($element['target_id']) ? $element['target_id'] : FALSE;
  }

  /**
   * Mass payment form sets cardinality of this field on the fly.
   */
  public function cardinalitySetMultiple() {
    $this->fieldDefinition->getFieldStorageDefinition()->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);
  }

}
